# loglib: easy player and addon logging
import es
import sys
import time
import path


# server vars
sv_logpath = es.ServerVar("loglib_directory", "logs/loglib", "Location for loglib to store it's logs")
eventscripts_gamedir = es.ServerVar("eventscripts_gamedir")
sv_loglvl = es.ServerVar("loglib_level", 1, "The logging level")

# set up some paths
logpath = path.path(eventscripts_gamedir)
logpath = logpath / str(sv_logpath)
playerpath = logpath / "player_logs"

if not logpath.exists():
    logpath.mkdir(1)
if not playerpath.exists():
    playerpath.mkdir(1)
    
# some global variables and so on
logged_players = {}
logged_addons = {}
 

class Logfile(object):
    '''
        A log file object: manages writing to a specific file
    '''
    def __init__(self,p):
        # sort out the log file location
        self.log = p
        
    def write(self,lvl,messages):
        if int(lvl) <= int(sv_loglvl):
            if "list" not in str(type(messages)):
                messages = [messages]
            lines = []
            for message in messages:
                lines.append("[%s]    %s" % (time.strftime("%X"),message))
            if not self.log.exists():
                # init the file
                self._write_log_headers()
            # and write the stuff
            self.log.write_lines(lines,append=True)
           
    def _write_log_headers(self):
        # write an initial header to the file
        self.log.write_lines(["############## Log File Started %s ##############" % (time.strftime("%c"))," "])
            
class AddonLogfile(object):
    def __init__(self,p):
        # some vars
        self.logs = {}
        # sort out the log file location
        self.path = logpath / p
        if not self.path.exists():
            self.path.mkdir(1)
        # now add the main log file
        self.log_date = time.strftime("%d-%b")
        self.setDefault(self.log_date)
        
    def setDefault(self,filename):
        self.addFile(filename)
        self.default = self.logs[filename]
        
    def addFile(self,filename):  
        if filename not in self.logs:
            # add the file
            self.logs[filename] = Logfile(self.path / ("%s.log" % filename))
            
    def write(self,lvl,messages):
        # quickly check that were aren't on a new day :)
        if self.log_date != time.strftime("%d-%b"):
            self.log_date = time.strftime("%d-%b")
            self.setDefault(self.log_date)
        self.default.write(lvl,messages)
           
    def __getitem__(self,filename):
        '''
            Allows the calling of <addon instance>[file].write()
        '''
        if filename not in self.logs:
            self.addFile(filename)
        return self.logs[filename]
        
class PlayerLogfile(AddonLogfile):
    '''
        Subclass AddonLogFile as we can use most of it directly!
    '''
    def __init__(self,p):
        # some vars
        self.logs = {}
        # sort out the log file location
        self.path = playerpath
        # now add the main log file
        self.log_date = time.strftime("%d-%b")
        self.setDefault(p.replace(":","-"))
    
    def write(self,lvl,messages):
        # quickly check that were aren't on a new day :)
        if self.log_date != time.strftime("%d-%b"):
            self.log_date = time.strftime("%d-%b")
            self.default.write(0,[" ","############## %s ##############" % (time.strftime("%c"))," "])
        self.default.write(lvl,messages)
# public methods
def addonLog(addon):
    '''
        Open a log file for the addon
    '''
    if not isLogged(addon):
        logged_addons[addon] = AddonLogfile(addon)
    return logged_addons[addon]
    
    
def togglePlayer(uniqueid):
    '''
        Toggle whether a player is logged or not
    '''
    # remember to replace the : in players steamid with dashes
    if isLogged(uniqueid):
        logged_players.pop(uniqueid,0)
    else:
        logged_players[uniqueid] = PlayerLogfile(uniqueid.replace(":","-"))
        return logged_players[uniqueid]
 
def playerLog(uniqueid):
    '''
        Opens a log file for a player if they are beign logged
    '''
    if isLogged(uniqueid):
        return logged_players[uniqueid]
    return None
    
def plog(uniqueid,lvl,messages):
    if isLogged(uniqueid):
        logged_players[uniqueid].write(lvl,messages)
    else:
        es.dbgmsg(0,"Sorry %s is not being logged" % uniqueid)
        
def log(addon,lvl,messages):
    if not isLogged(addon):
        addonLog(addon)
    logged_addons[addon].write(lvl,messages)
        
def isLogged(id):
    '''
        Check if a player or addon is logged
    '''
    id = str(id)
    if id.startswith("STEAM") and id in logged_players:
        # steamid & it exists
        return True
    elif id in logged_addons:
        # addon
        return True
    else:
        return False